package cn.estsh.i3plus.core.apiservice.serviceimpl.base.login.strategy;

import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginService;
import cn.estsh.i3plus.core.api.iservice.base.ISystemLoginStrategyService;
import cn.estsh.i3plus.core.api.iservice.busi.ILicenseClickService;
import cn.estsh.i3plus.core.api.iservice.busi.ISysUserService;
import cn.estsh.i3plus.platform.common.util.CommonConstWords;
import cn.estsh.i3plus.pojo.base.bean.BaseThreadLocal;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.model.platform.SysLoginModel;
import cn.estsh.i3plus.pojo.platform.bean.SessionUser;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.util.ImppRedis;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.function.BiFunction;

import static cn.estsh.i3plus.platform.common.util.CommonConstWords.DEFAULT_LANGUAGE;

/**
 * @author Wynne.Lu
 * @date 2021/1/18 下午4:39
 * @desc
 */
@Slf4j
@Service
public class DefaultLoginStrategy implements ISystemLoginStrategyService {

    @Resource(name = "redisCore")
    protected ImppRedis redisCore;

    @Autowired
    private ILicenseClickService licenseClickService;

    @Autowired
    private ISysUserService userService;

    @Autowired
    private ISystemLoginService loginService;

    @Override
    public BiFunction<HttpServletRequest, SysLoginModel, ResultBean> login() {
        return (request, loginModel) -> {
            ResultBean result;
            ValidatorBean.checkNotNull(loginModel.getLanguageCode(), "语言不能为空");
            // TODO 后期移除，暂时用于避免自动登录后前台没有正确的传输组织代码信息
            if ("null".equals(loginModel.getLanguageCode())) {
                loginModel.setLanguageCode(DEFAULT_LANGUAGE);
            }

            // 设置语言代码
            BaseThreadLocal.setData(BaseThreadLocal.LANGUAGE_CODE, loginModel.getLanguageCode());

            ValidatorBean.checkNotNull(loginModel.getLoginName(), "用户名不能为空");
            ValidatorBean.checkNotNull(loginModel.getLoginPwd(), "密码不能为空");

            int sessionMode = RedisCacheTool.getSysConfigIntVal(CommonConstWords.CONFIG_SESSION_MODE, CommonConstWords.CONFIG_SESSION_MODE_DEFAULT);
            if (sessionMode != CommonEnumUtil.SESSION_MODE.SEIZE.getValue()) {
                AuthUtil.logout();
            }

            SessionUser user = userService.loginUser(
                    loginModel.getLoginName().trim(),
                    loginModel.getLoginPwd().trim(),
                    loginModel.getLanguageCode(),
                    loginService.getLoginPlatform(request).getValue(),
                    loginModel.getDeviceId()
            );

            String redisKey = CommonConstWords.USER_LOGIN_ERROR + "_" + user.getUser().getId();
            redisCore.deleteKey(redisKey);

            result = new ResultBean(true, AuthUtil.getSession().getId().toString(), AuthUtil.getSessionUser());
            result.setUrl("/");

            AuthUtil.setOrganize(user.getUser().getOrganize());
            return result;
        };
    }
}
